@@ -229,7 +229,7 @@ _on_discovered_cb (GstDiscoverer *discoverer, GstDiscovererInfo *info, GError *e
 
 	result = gst_discoverer_info_get_result (info);
 	if (GST_DISCOVERER_OK != result) {
-		g_warning ("Could not read metadata from %s\n", uri);
+		g_warning ("Could not read metadata from %s", uri);
 		goto done;
 	}
 
@@ -262,7 +262,7 @@ _on_finished_cb (GstDiscoverer *discoverer, AVMetaReaderGstPrivate *priv)
 }
 
 static gboolean
-_read (AVMetaReader *_reader, DmapAvRecord *record, const gchar *path)
+_read (AVMetaReader *_reader, DmapAvRecord *record, const gchar *path, GError **error)
 {
 	AVMetaReaderGst *reader = AV_META_READER_GST (_reader);
 
@@ -272,18 +272,15 @@ _read (AVMetaReader *_reader, DmapAvRecord *record, const gchar *path)
 	g_assert (NULL != path);
 
 	gboolean fnval = FALSE;	
-	GError *error = NULL;
 	gchar *uri = NULL;
 
-	uri = g_filename_to_uri (path, NULL, NULL);
+	uri = g_filename_to_uri (path, NULL, error);
 	if (NULL == uri) {
-		g_warning ("Error converting %s to URI", path);
 		goto done;
 	}
 
-	reader->priv->discoverer = gst_discoverer_new (_DISCOVER_TIMEOUT, &error);
-	if (NULL != error) {
-		g_warning ("Error creating discoverer instance: %s\n", error->message);
+	reader->priv->discoverer = gst_discoverer_new (_DISCOVER_TIMEOUT, error);
+	if (NULL != *error) {
 		goto done;
 	}
 
@@ -293,7 +290,10 @@ _read (AVMetaReader *_reader, DmapAvRecord *record, const gchar *path)
 	gst_discoverer_start (reader->priv->discoverer);
 
 	if (! gst_discoverer_discover_uri_async (reader->priv->discoverer, uri)) {
-		g_warning ("Failed to start discovering URI '%s'\n", uri);
+		g_set_error(error,
+		            DMAP_ERROR,
+		            DMAP_STATUS_FAILED,
+		           "Failed to start discovering URI '%s'", uri);
 		goto done;
 	}
 
@@ -308,10 +308,6 @@ done:
 		g_free (uri);
 	}
 
-	if (NULL != error) {
-		g_error_free (error);
-	}
-
 	if (reader->priv->discoverer) {
 		g_object_unref (reader->priv->discoverer);
 	}
@@ -347,8 +343,6 @@ static void av_meta_reader_gst_class_init (AVMetaReaderGstClass *klass)
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 	AVMetaReaderClass *av_meta_reader_class = AV_META_READER_CLASS (klass);
 
-	g_type_class_add_private (klass, sizeof (AVMetaReaderGstPrivate));
-
 	gobject_class->set_property = _set_property;
 	gobject_class->get_property = _get_property;
 	gobject_class->finalize = _finalize;
@@ -357,4 +351,8 @@ static void av_meta_reader_gst_class_init (AVMetaReaderGstClass *klass)
 	av_meta_reader_class->get_option_group = _get_option_group;
 }
 
-G_DEFINE_DYNAMIC_TYPE (AVMetaReaderGst, av_meta_reader_gst, TYPE_AV_META_READER)
+G_DEFINE_DYNAMIC_TYPE_EXTENDED (AVMetaReaderGst,
+                                av_meta_reader_gst,
+                                TYPE_AV_META_READER,
+                                0,
+                                G_ADD_PRIVATE_DYNAMIC (AVMetaReaderGst))
