@@ -295,14 +295,14 @@ _set_from_blob (DmapRecord *_record, GArray *blob)
 
 	size = *((guint *) ptr);
 	if (DMAP_HASH_SIZE != size) {
-		g_warning ("Improper hash size in cache\n");
+		g_warning ("Improper hash size in cache");
 		goto _done;
 	}
 	ptr += sizeof (size);
 
 	hash = g_array_sized_new (FALSE, FALSE, 1, size);
 	if (NULL == hash) {
-                g_warning ("Error allocating array for hash\n");
+                g_warning ("Error allocating array for hash");
                 goto _done;
         }
 	g_array_append_vals (hash, ptr, size);
@@ -326,7 +326,7 @@ _set_from_blob (DmapRecord *_record, GArray *blob)
 	if (size) {
 		thumbnail = g_array_sized_new (FALSE, FALSE, 1, size);
 		if (NULL == thumbnail) {
-			g_warning ("Error allocating array for thumbnail\n");
+			g_warning ("Error allocating array for thumbnail");
 			goto _done;
 		}
 		g_array_append_vals (thumbnail, ptr, size);
@@ -352,26 +352,26 @@ _set_from_blob (DmapRecord *_record, GArray *blob)
 
 	file = g_file_new_for_uri (location);
 	if (NULL == file) {
-		g_warning ("Could not open %s\n", location);
+		g_warning ("Could not open %s", location);
 		goto _done;
 	}
 
         stream = g_file_read (file, NULL, &error);
         if (NULL == stream) {
-                g_warning ("Could not read %s: %s\n", location, error->message);
+                g_warning ("Could not read %s: %s", location, error->message);
                 goto _done;
         }
 
 	if (FALSE /* FIXME: calculate_media_hashes */) {
                 if (! util_hash_file (location, hash2)
                  || memcmp (hash->data, hash2, DMAP_HASH_SIZE)) {
-                        g_warning ("Media file hash has changed since being cached\n");
+                        g_warning ("Media file hash has changed since being cached");
                         goto _done;
                 }
         } else {
                 path = g_file_get_path(file);
                 if (NULL == path) {
-                        g_warning ("Could not convert %s to local path; assuming modified\n", location);
+                        g_warning ("Could not convert %s to local path; assuming modified", location);
                         goto _done;
                 }
                 if (stat (path, &buf) == -1) {
@@ -379,7 +379,7 @@ _set_from_blob (DmapRecord *_record, GArray *blob)
                         goto _done;
                 } else {
                         if (buf.st_mtime > mtime) {
-                                g_warning ("Media file mtime has changed since being cached\n");
+                                g_warning ("Media file mtime has changed since being cached");
                                 goto _done;
                         }
                 }
@@ -420,14 +420,6 @@ _done:
                 g_object_unref (stream);
         }
 
-	if (NULL != hash) {
-		g_array_unref (hash);
-	}
-
-	if (NULL != thumbnail) {
-		g_array_unref (thumbnail);
-	}
-
 	return fnval;
 }
 
@@ -442,8 +434,6 @@ static void dmapd_dpap_record_class_init (DmapdDmapImageRecordClass *klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-	g_type_class_add_private (klass, sizeof (DmapdDmapImageRecordPrivate));
-
 	gobject_class->set_property = _set_property;
 	gobject_class->get_property = _get_property;
 	gobject_class->finalize = _finalize;
@@ -494,7 +484,8 @@ _dmap_record_iface_init (gpointer iface, gpointer data)
 
 G_DEFINE_TYPE_WITH_CODE (DmapdDmapImageRecord, dmapd_dpap_record, G_TYPE_OBJECT, 
 			 G_IMPLEMENT_INTERFACE (DMAP_TYPE_IMAGE_RECORD, _dmap_image_record_iface_init)
-			 G_IMPLEMENT_INTERFACE (DMAP_TYPE_RECORD, _dmap_record_iface_init))
+			 G_IMPLEMENT_INTERFACE (DMAP_TYPE_RECORD, _dmap_record_iface_init)
+                         G_ADD_PRIVATE (DmapdDmapImageRecord))
 
 static void
 _finalize (GObject *object)
@@ -522,7 +513,7 @@ _finalize (GObject *object)
 }
 
 DmapdDmapImageRecord *
-dmapd_dpap_record_new (const char *path, gpointer reader)
+dmapd_dpap_record_new (const char *path, gpointer reader, GError **error)
 {
 	DmapdDmapImageRecord *record = NULL;
 	guchar hash_buf[DMAP_HASH_SIZE];
@@ -531,21 +522,19 @@ dmapd_dpap_record_new (const char *path, gpointer reader)
 	GArray *hash = NULL;
 
 	if (path) {
-		location = g_filename_to_uri (path, NULL, NULL);
+		location = g_filename_to_uri (path, NULL, error);
                 if (NULL == location) {
-                        g_warning ("Error converting %s to URI\n", path);
                         goto _done;
                 }
 
 		hash = g_array_sized_new (FALSE, FALSE, 1, DMAP_HASH_SIZE);
-                if (NULL == hash) {
-                        g_warning ("Error allocating memory for record's hash field\n");
-                        goto _done;
-                }
 
 		if (FALSE /* FIXME: calculate_media_hashes */) {
 			if (! util_hash_file (location, hash_buf)) {
-				g_warning ("Unable to hash %s\n", location);
+				g_set_error(error,
+				            DMAP_ERROR,
+				            DMAP_STATUS_FAILED,
+				           "Unable to hash %s", location);
 				goto _done;
 			}
 		} else {
@@ -555,10 +544,6 @@ dmapd_dpap_record_new (const char *path, gpointer reader)
 		g_array_append_vals (hash, hash_buf, DMAP_HASH_SIZE);
 
 		record = DMAPD_DPAP_RECORD (g_object_new (TYPE_DMAPD_DPAP_RECORD, NULL));
-		if (NULL == record) {
-                        g_warning ("Error allocating memory for record\n");
-                        goto _done;
-                }
 
 		if (stat (path, &buf) == -1) {
                         g_warning ("Unable to determine mtime of %s", path);
@@ -573,27 +558,20 @@ dmapd_dpap_record_new (const char *path, gpointer reader)
                                       "hash",     hash,
 		                       NULL);
 
-		if (! photo_meta_reader_read (PHOTO_META_READER (reader), DMAP_IMAGE_RECORD (record), path)) {
+		if (! photo_meta_reader_read (PHOTO_META_READER (reader),
+		                              DMAP_IMAGE_RECORD (record),
+		                              path,
+		                              error)) {
 			g_object_unref (record);
 			record = NULL;
 			goto _done;
 		}
 	} else {
 		record = DMAPD_DPAP_RECORD (g_object_new (TYPE_DMAPD_DPAP_RECORD, NULL));
-		if (NULL == record) {
-                        g_warning ("Error allocating memory for record\n");
-                        goto _done;
-                }
 	}
 
 _done:
-	if (NULL != location) {
-                g_free (location);
-        }
-
-        if (NULL != hash) {
-                g_array_unref (hash);
-        }
+	g_free (location);
 
 	return record;
 }
