@@ -858,7 +858,7 @@ main (int argc, char *argv[])
 						NULL));
 			share_def->share = _serve(DAAP, factory, share_def, &error);
 			if (NULL == share_def->share) {
-				g_error("Error serving: %s\n", error->message);
+				g_error("Error serving: %s", error->message);
 			}
 #else
 		g_error ("DAAP support not present");
@@ -877,7 +877,7 @@ main (int argc, char *argv[])
 						NULL));
 			share_def->share = _serve(DPAP, factory, share_def, &error);
 			if (NULL == share_def->share) {
-				g_error("Error serving: %s\n", error->message);
+				g_error("Error serving: %s", error->message);
 			}
 #else
 			g_error ("DPAP support not present");
