@@ -59,7 +59,8 @@ struct _PhotoMetaReaderClass {
 
 	gboolean      (*read) (PhotoMetaReader *reader,
                                DmapImageRecord *record,
-                               const gchar *path);
+                               const gchar *path,
+                               GError **error);
 	GOptionGroup *(*get_option_group) (PhotoMetaReader *reader);
 };
 
@@ -67,7 +68,8 @@ GType       photo_meta_reader_get_type      (void);
 
 gboolean photo_meta_reader_read (PhotoMetaReader *reader,
                                  DmapImageRecord *record,
-                                 const gchar *path);
+                                 const gchar *path,
+                                 GError **error);
 
 GOptionGroup *photo_meta_reader_get_option_group (PhotoMetaReader *reader);
 
